// BePrint 1.000 by Earl Colby Pottinger. 
// Partly based on code found in the Beos CDROM 
// Partly based on instructions found in "Programming the Be Operating System" By Dan Parks Sydow. 
// Additional help by Larry Baydak in debugging and coding/commenting style. 
// Stripped down version of BePrinter to just print ASSCI files.

#include "BePrint_Includes.h"
#include "BePrint_Menus.h"
#include "BePrint_Defaults.h"

//Constants needed. 
const char *APP_SIGNATURE="application/x-vnd.BePrint"; // Application's signature.

//Variables needed to control printer.
char PrintCPI[99];                   // Holds control string to set printer's characters per inch.
char PrintQuality[99],    PrintLQ[99];     // Control string to set printer's text quality, text of quality mode.
char PrintSpacing[99],    PrintSpace[99];  // Control string to set character spacing, text of spacing.
char PaperTray[99],       PaperSource[99]; // Control string to set which paper tray to use, text of tray's name.
char PrintOrientation[99],PrintOrient[99]; // Control string to set paper orientation, text of orientation.
char PrinterWrap[99],     PrinterWRAP[99];   // Control string to set printer's line wrap control, text of print's line wrap.
char PrintDirection[99],  PrintDir[99];    // Control string to set printhead movements, text of mode.

double PortraitX,  PortraitY;  // Width & Height in inches.
double LandscapeX, LandscapeY; // Width & Height in inches
double X, Y; // Speeds drawing to screen. 
int ASC;

// File and Print buffers.
FILE *PrinterPort;       // Handle for printer port.
long n;
long FileLenght=0;       // Number of bytes in file input buffer.
char FileBuffer[999999]; // Buffers file to print. (Temp solution).
char TextBuffer[999];    // Text buffer to convert number variables.  Print lines in columns

// Objects needed.
BFile      FileObject;     // BFile object
BFilePanel *OpenFilePanel; // Define object to create open panels from.
BAlert     *alert;         // Create a pointer (*alert) to an alert window
 long       result=0;      // Returned value from the BAlert::Go() member function.

// BePrintView.h
class BePrintView : public BView {                   // Define the class object BePrintView.
 public:       BePrintView(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void AttachedToWindow();                   // Called by the OS when 'aWindow->AddChild(aView)' used.
  virtual void Draw(BRect updateRect);               // Called by the OS when 'aWindow->Show()' used.
   inline void NextLine() { X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); }
} ;

// BePrintView.cpp
BePrintView::BePrintView(BRect rect, char *name) // Define 'BePrintView' constructor call.
 : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) {    //
} ;

void BePrintView::AttachedToWindow() { // Sets the character of the view when a view ia attached to a window.
 SetFont(be_plain_font);  // Set font outline to bold.
 SetFontSize(12);         // Set font size.
} ;

void BePrintView::Draw(BRect updateRect) { // Draws in view when show is called.
 X=0.0; Y=0.0; // Move pen to top of window.
 NextLine(); sprintf(TextBuffer,"Print's Linewrapping: <%s>",PrinterWRAP);   DrawString(TextBuffer); // Print setting for line wrapping.
 NextLine(); sprintf(TextBuffer,"Print Data Path: <%s>",PrinterPath);        DrawString(TextBuffer); // Where data is sent after processing.
}

// BePrintWindow.h
class BePrintWindow : public BWindow {           // Define the class object BePrintWindow.
 public:       BePrintWindow(BRect frame);       // Constructor for a window thread.
  virtual bool QuitRequested();                    // Add function is close box is clicked.
  virtual void MessageReceived(BMessage *message); // Setup to recieve menu & windows messages.
 private:
  BMenuBar     *menubar;                           // Define object to hold all menu structures.
 
} ;

// BePrintWindow.cpp
BePrintWindow::BePrintWindow(BRect frame) // Define 'BePrintWindow' constructor call.
 : BWindow(frame,                   // Frame area of the window
           "BePrint Window",      // Name of the window, appears in the Tab on top of the window.
           B_TITLED_WINDOW,         // Type of window (Look & Feel)
           B_NOT_RESIZABLE )        // Behavior of the window (resize knob, etc)
{
 BRect rMenu;                       // Set the Window Bpoints into the rectangle object(aRect) to define a MenuBar.
 BMenu *menu;                       // 
 rMenu = Bounds();                  // Find active area of view.
 menubar = new BMenuBar(rMenu,      // Invoke a new dynamically allocated menubar object inside the window
          "printer menu_bar");      // by calling the constructor. Plus supply a ID name for the menu.
 AddChild(menubar);                 // Add menubar to window created.
 
 menu = new BMenu("File"); // Adds File menu to menu bar.
 menu->AddItem(new BMenuItem("Page Setup",new BMessage(MENU_FILE_SETUP)     )); // Setup page margins.
 menu->AddItem(new BMenuItem("Print",     new BMessage(MENU_FILE_PRINT), 'P')); // Print a flie or clipping.
 menu->AddItem(new BMenuItem("Quit",      new BMessage(MENU_FILE_QUIT),  'Q')); // Exit program.
 menubar->AddItem(menu);

 menu = new BMenu("Prints"); // Adds Print menu to menu bar
 menu->AddItem(new BMenuItem("Parallel Port 1", new BMessage(MENU_PRINTER_PAR1))); // Print's Parallel port 1 interface.
 menu->AddItem(new BMenuItem("Parallel Port 2", new BMessage(MENU_PRINTER_PAR2))); // Print's Parallel port 2 interface.
 menu->AddItem(new BMenuItem("Serial Port 1",   new BMessage(MENU_PRINTER_SER1))); // Print's Serial port 1 interface.
 menu->AddItem(new BMenuItem("Serial Port 2",   new BMessage(MENU_PRINTER_SER2))); // Print's Serial port 2 interface.
 menu->AddItem(new BMenuItem("Print File",    new BMessage(MENU_PRINTER_PRNT))); // Print's Print to File.
 menubar->AddItem(menu);
 
 menu = new BMenu("Print Modes"); // Adds Print mode menu to menu bar
 menu->AddItem(new BMenuItem("Long Lines Truncated",    new BMessage(MENU_MODE_LINE_T))); // Truncate extra long lines of text.
 menu->AddItem(new BMenuItem("Long Lines LineWrapped",  new BMessage(MENU_MODE_LINE_L))); // LineWrap extra long lines of text.
 menu->AddItem(new BMenuItem("Long Lines Word Wrapped", new BMessage(MENU_MODE_LINE_W))); // WordWrap extra long lines of text.
 menubar->AddItem(menu);
 Show();
} ;

bool BePrintWindow::QuitRequested() { // Close button on window pressed. A Blooper call to Bwindows. 
 be_app->PostMessage(B_QUIT_REQUESTED); // Send a message to the global variable 'be_app' to tell the program to quit.
 return(true);                          // Tell the system (Blooper) to go ahead and close window.
} ;

// BePrintWindow::MessageReceived.  Called when a message is received by our application.
void BePrintWindow::MessageReceived(BMessage *message) {
 switch(message->what) {
  case MENU_FILE_SETUP: {
   alert = new BAlert("Print Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Setup page margins.

  case MENU_FILE_PRINT: { // Print a file or clipping to the output port.
   OpenFilePanel = new BFilePanel(B_OPEN_PANEL); // Create the open a file panel.
   OpenFilePanel->Show(); // Display open a file panel, get file name to print.   
   break; } ;             // End of case

  case MENU_FILE_QUIT: {  // Exit the program.
  if (PrinterPort!=0) {   // Test if printer port is open.
   fclose(PrinterPort);   // If it is, close it.
   PrinterPort=NULL ; } ; // Clear the handle to show it is closed.
  be_app->PostMessage(B_QUIT_REQUESTED);
  break; } ;              // Exit program, sends a message to BApplication.

  case MENU_PRINTER_PAR1: {
   PrinterPath = "/dev/parallel/parallel1"; // Print's Parallel port 1 interface.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_PRINTER_PAR2: {
   PrinterPath = "/dev/parallel/parallel2"; // Print's Parallel port 2 interface.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_PRINTER_SER1: {
   PrinterPath = "/dev/serial/serial1"; // Print's Serial port 1 interface.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_PRINTER_SER2: {
   PrinterPath = "/dev/serial/serial2"; // Print's Serial port 2 interface.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_PRINTER_PRNT: {
   PrinterPath = "PrintFile"; // Print's Print to File.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  

  case MENU_MODE_LINE_T: {
   LineMode="Truncated"; // Truncate extra long lines of text.
   alert = new BAlert("Print Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_MODE_LINE_L: {
   LineMode="LineWrap"; // LineWrap extra long lines of text.
   alert = new BAlert("Print Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  case MENU_MODE_LINE_W: {
   LineMode="WordWarp"; // WordWrap extra long lines of text.
   alert = new BAlert("Print Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Minimize(true); Show(); Minimize(false); break; } ; // Redraw window with the view attached.  
  
  default:              { BWindow::MessageReceived(message); break; } ;
} ; }

// BePrintWorld.h
class BePrintApplication : public BApplication {   // Define the class object BePrintApplication.
 public:       BePrintApplication();               // Constructor for an application thread.
  virtual void MessageReceived(BMessage *message); // ??????????????
  virtual void RefsReceived(BMessage *message);    // Recieve messages from file panel.
} ;

// BePrintWorld.cpp  The main() function's only real job in a basic BeOS application is to create the BApplication object and run it.
int main(int, char**) {
 BePrintApplication myApplication; // Create an Be_app instance?
 myApplication.Run();              // Run the app thread.  Start to process messages.
 return(0);                        // Return 0 for no errors.
} ;

BePrintApplication::BePrintApplication() : BApplication(APP_SIGNATURE) { // Define 'BePrintApplication' constructor call. 
 BePrintWindow *aWindow;                // Create a pointer(*aWindow) to an object(BePrintWindow).
 BePrintView   *aView;                  // Create a pointer(*aView) to an object(BePrintView).
 BRect           aRect;                 // Set up a rectangle object(aRect). Size of Window to open.

// Create a window. 
 aRect.Set(32, 32, 600, 600);        // Set the Screen Bpoints into the rectangle object(aRect) to define a window.
 aWindow = new BePrintWindow(aRect); // Invoke a new dynamically allocated window object by calling the constructor.
 aWindow->Show();                    // Make window visible.

// Create a view.
 aRect.Set(0, 19, 599, 599);        // Set the Window Bpoints into the rectangle object(aRect) to define a view.
 aView = new BePrintView(aRect,     // Invoke a new dynamically allocated view object inside the window
         "BePrint_View_ID_Name" ) ; // by calling the constructor. Plus supply a ID name for the view.
 aWindow->AddChild(aView);          // Add view to window.
 aWindow->Show();                   // Redraw window with the view attached.

} ;

// BePrintApp::MessageReceived.  Handle incoming messages. 
void BePrintApplication::MessageReceived(BMessage *message) { // ???????????????
 switch(message->what) {                                        // ???????????????
  default: { BApplication::MessageReceived(message); break; }   // ???????????????
} ; }

 void StartPrinting(); 
 
// BePrintApp::RefsReceived.  Handle a refs received message.
// The message specifies
// where to get it (see BFilePanel in the Storage Kit chapter of the Be Book).
void BePrintApplication::RefsReceived(BMessage *message) {
 entry_ref  ref;        // The entry_ref to open, For the directory to save into.
 status_t   err;        // The error/return code
 int32      ref_num;    // The index into the ref list
 off_t      length;     // Length of file.

// Open printer port.
 PrinterPort=fopen(PrinterPath,"w");      // Open file handle to printer.
 if (PrinterPort==0) {                    // Test if port opening failed.
  alert = new BAlert("PrintAlert",      // If yes, then display Alert box.  Alert Title.
                     "Port failed",       //could not be opened, Continue program?",
                     "OK",                // Button #0 text. Do not quit program.
                     "Quit");             // Button #1 text. Do quit program.            
  result = alert->Go(); }                 // Get which button was pushed.
 if (result==1) {                         // Exit program choosen.
  fclose(PrinterPort);                    // Close the printer port.
  be_app->PostMessage(B_QUIT_REQUESTED);  // Sends a quit message to BApplication.
  return; } ;                             // Exit program, returns to BApplication.

 ref_num = 0; // Loop through the ref list and open each one
 do {
     err = message->FindRef("refs", ref_num, &ref);      // No more reference so exit.
     if (err != B_OK) { fclose(PrinterPort); return; } ; // If error setting file exit.
     err = FileObject.SetTo(&ref, B_READ_ONLY);          // Set BFile object to file to print.
     if (err != B_OK) { fclose(PrinterPort); return; } ; // If error setting file exit.
     FileObject.GetSize(&length);                        // Get size of file to print.
     FileLenght = FileObject.Read(FileBuffer, length);   // Read file to be printed into printer.  
     for (n=0;n<FileLenght;n++) {                        // Process entire file.
          ASC=(int)FileBuffer[n];                        // Get a single character.
          if (ASC>127) { ASC=64; };
          if (ASC<0) { ASC=64; };
          FileBuffer[n]=(char)ASC;                       // Write one byte into buffer to send to printer.
     }//ENDIF
     err=fwrite(FileBuffer,1,FileLenght,PrinterPort);    // err contains number of items written.
     ref_num ++;                                         // Index to next reference.
    } 
    while (1);                                           // Loop continuously.
 }
